@@ -19,8 +19,9 @@ class BaseAlgorithm:
         self.library_version = None
         self.model = None
         self.uid = params.get("uid", str(uuid.uuid4()))
+        self.ml_task = params.get("ml_task")
 
-    def fit(self, X, y):
+    def fit(self, X, y, X_validation=None, y_validation=None, log_to_file=None):
         pass
 
     def predict(self, X):
@@ -29,7 +30,7 @@ class BaseAlgorithm:
     # needed for feature importance
     def predict_proba(self, X):
         y = self.predict(X)
-        if "num_class" in self.params:
+        if "num_class" in self.params and self.params["num_class"] > 2:
             return y
         return np.column_stack((1 - y, y))
 
