@@ -6,26 +6,27 @@ import time
 import numpy as np
 import pandas as pd
 import logging
+from tabulate import tabulate
 
-from supervised.model_framework import ModelFramework
+from supervised.algorithms.registry import AlgorithmsRegistry
+from supervised.algorithms.registry import BINARY_CLASSIFICATION
+from supervised.algorithms.registry import MULTICLASS_CLASSIFICATION
+from supervised.algorithms.registry import REGRESSION
 from supervised.callbacks.early_stopping import EarlyStopping
 from supervised.callbacks.metric_logger import MetricLogger
 from supervised.callbacks.learner_time_constraint import LearnerTimeConstraint
 from supervised.callbacks.total_time_constraint import TotalTimeConstraint
-
-from supervised.utils.metric import Metric
-from supervised.algorithms.registry import AlgorithmsRegistry
-from supervised.algorithms.registry import (
-    BINARY_CLASSIFICATION,
-    MULTICLASS_CLASSIFICATION,
-    REGRESSION,
-)
-from supervised.tuner.mljar_tuner import MljarTuner
 from supervised.ensemble import Ensemble
+from supervised.exceptions import AutoMLException
+from supervised.model_framework import ModelFramework
+from supervised.preprocessing.exclude_missing_target import ExcludeRowsMissingTarget
+from supervised.tuner.data_info import DataInfo
+from supervised.tuner.mljar_tuner import MljarTuner
 from supervised.utils.additional_metrics import AdditionalMetrics
+from supervised.utils.config import mem
 from supervised.utils.config import LOG_LEVEL
 from supervised.utils.leaderboard_plots import LeaderboardPlots
-from supervised.preprocessing.exclude_missing_target import ExcludeRowsMissingTarget
+from supervised.utils.metric import Metric
 
 logging.basicConfig(
     format="%(asctime)s %(name)s %(levelname)s %(message)s", level=logging.ERROR
@@ -33,13 +34,6 @@ logging.basicConfig(
 logger = logging.getLogger(__name__)
 logger.setLevel(LOG_LEVEL)
 
-from supervised.exceptions import AutoMLException
-from supervised.tuner.data_info import DataInfo
-import gc
-from supervised.utils.config import mem
-
-from tabulate import tabulate
-
 
 class AutoML:
     """
@@ -47,54 +41,47 @@ class AutoML:
     """
 
     def __init__(
-        self,
-        results_path=None,
-        total_time_limit=60 * 60,
-        model_time_limit=None,
-        algorithms=[
-            "Baseline",
-            "Linear",
-            "Decision Tree",
-            "Random Forest",
-            "Extra Trees",
-            "LightGBM",
-            "Xgboost",
-            "CatBoost",
-            "Neural Network",
-            "Nearest Neighbors",
-        ],
-        tuning_mode="Normal",
-        train_ensemble=True,
-        stack=True,
-        optimize_metric=None,
-        validation={
-            "validation_type": "kfold",
-            "k_folds": 10,
-            "shuffle": True,
-            "stratify": True,
-        },
-        verbose=True,
-        ml_task=None,
-        explain_level=2,
-        seed=1,
+        self, results_path=None, total_time_limit=30 * 60, mode="Explain", **kwargs
     ):
+
         """
-        Create the AutoML object. Initialize directory for results.
-
-        :param results_path: The path where all results will be saved. 
-        If left `None` then the name of directory will be generated, with schema: AutoML_{number},
-        where number can be from 1 to 100 - depends which direcory name will be available.
-
-        If the `results_path` will point to directory with AutoML results, then all models will be loaded.
+        Initialize the AutoML object. 
         
-        :param total_time_limit: The time limit in seconds for AutoML training. It is not used when `model_time_limit` is not `None`.
+        :param results_path: The path with results. 
+        If left `None` then the name of directory will be generated, with template: AutoML_{number},
+        where the number can be from 1 to 1,000 - depends which direcory name will be available.
+
+        If the `results_path` will point to directory with AutoML results (`params.json` must be present), 
+        then all models will be loaded.
         
-        :param model_time_limit: The time limit in seconds for training single model. 
+        :param total_time_limit: The time limit in seconds for AutoML training. 
+        It is not used when `model_time_limit` is not `None`.
+        
+        :params mode
+
+        ## Additional (optional parameters)
+
+        :param model_time_limit: The time limit for training a single model, in seconds. 
         If `model_time_limit` is set, the `total_time_limit` is not respected. 
-        Single model can contain several learners, for example in the case of 10-fold cross-validation, one model will have 10 learners.
-        Based on `model_time_limit` the time limit for single learner is computed.
-        
-        :param algorithms: The list of algorithms that will be used in the training.
+        The single model can contain several learners.
+        The time limit for single learner is computed based on `model_time_limit`.
+
+        For example, in the case of 10-fold cross-validation, one model will have 10 learners. 
+        The `model_time_limit` is time for all 10 learners.
+
+        :param algorithms: The list of algorithms that will be used in the training. The algorithms can be:
+            [
+                "Baseline",
+                "Linear",
+                "Decision Tree",
+                "Random Forest",
+                "Extra Trees",
+                "LightGBM",
+                "Xgboost",
+                "CatBoost",
+                "Neural Network",
+                "Nearest Neighbors",
+            ]
         
         :param tuning_mode: The mode for tuning. It can be: `Normal`, `Sport`, `Insane`, `Perfect`. The names are kept the same as in https://mljar.com application.
         
@@ -109,7 +96,7 @@ class AutoML:
         
         :param train_ensemble: If true then at the end of models training the ensemble will be created. (Default is `True`)
 
-        :param stack: If true then stacked models will be created. Stack level is 1. (Default is `True`)
+        :param stack_models: If true then stacked models will be created. Stack level is 1. (Default is `True`)
         
         :param optimize_metric: The metric to be optimized. (not implemented yet, please left `None`)
         
@@ -119,6 +106,7 @@ class AutoML:
         {"validation_type": "kfold", "k_folds": 5, "shuffle": True, "stratify": True, "random_seed": 123}
         ```
         :param verbose: Not implemented yet.
+
         :param ml_task: The machine learning task that will be solved. Can be: `"binary_classification", "multiclass_classification", "regression"`.
         If left `None` AutoML will try to guess the task based on target values. 
         If there will be only 2 values in the target, then task will be set to `"binary_classification"`.
@@ -129,44 +117,152 @@ class AutoML:
         `explain_level = 0` means no explanations
         `explain_level = 1` means produce importance plot (with permutation method), for decision trees produce tree plots, for linear models save coefficients
         `explain_level = 2` the same as for `1` plus SHAP explanations
-        :param seed: The seed for random generator.
         
+        :param seed: The seed for random generator.
+
         """
         logger.debug("AutoML.__init__")
 
-        # total_time_limit is the time for computing for all models
-        # model_time_limit is the time for computing a single model
-        # if model_time_limit is None then its value is computed from total_time_limit
-        # if total_time_limit is set and model_time_limit is set, then total_time_limit constraint will be omitted
+        self._results_path = results_path
+        """
+        total_time_limit is the time for computing for all models
+        model_time_limit is the time for computing a single model
+        if model_time_limit is None then its value is computed from total_time_limit
+        if total_time_limit is set and model_time_limit is set, then total_time_limit constraint will be omitted
+        """
         self._total_time_limit = total_time_limit
-        self._model_time_limit = model_time_limit
-        # time limit in seconds for single learner (model consists of learners)
-        # the value is computed before fit, initilize with any number
-        self._time_limit = 1
 
-        self._train_ensemble = train_ensemble
-        self._stack = stack
+        if mode not in ["Explain", "Perform", "Compete"]:
+            print("mode should be: Explain, Perform or Compete")
+            print("Setting mode=Explain")
+            mode = "Explain"
+
+        self._mode = mode
+
+        if self._mode == "Explain":
+            self._train_ensemble = True
+            self._stack_models = False
+            self._validation = {
+                "validation_type": "split",
+                "train_ratio": 0.75,
+                "shuffle": True,
+                "stratify": True,
+            }
+            self._algorithms = [
+                "Baseline",
+                "Linear",
+                "Decision Tree",
+                "Random Forest",
+                "Xgboost",
+                "Neural Network",
+            ]
+            self._explain_level = 2
+            self._start_random_models = 1
+            self._hill_climbing_steps = 0
+            self._top_models_to_improve = 0
+        elif self._mode == "Perform":
+            self._train_ensemble = True
+            self._stack_models = False
+            self._validation = {
+                "validation_type": "kfold",
+                "k_folds": 5,
+                "shuffle": True,
+                "stratify": True,
+            }
+            self._algorithms = [
+                "Linear",
+                "Random Forest",
+                "LightGBM",
+                "Xgboost",
+                "CatBoost",
+                "Neural Network",
+            ]
+            self._explain_level = 1
+            self._start_random_models = 5
+            self._hill_climbing_steps = 2
+            self._top_models_to_improve = 2
+        elif self._mode == "Compete":
+            self._train_ensemble = True
+            self._stack_models = True
+            self._validation = {
+                "validation_type": "kfold",
+                "k_folds": 10,
+                "shuffle": True,
+                "stratify": True,
+            }
+            self._algorithms = [
+                "Linear",
+                "Decision Tree",
+                "Random Forest",
+                "Extra Trees",
+                "LightGBM",
+                "Xgboost",
+                "CatBoost",
+                "Neural Network",
+                "Nearest Neighbors",
+            ]
+            self._explain_level = 0
+            self._start_random_models = 10
+            self._hill_climbing_steps = 2
+            self._top_models_to_improve = 3
+
+        self._model_time_limit = None
+        if "model_time_limit" in kwargs:
+            self._model_time_limit = kwargs["model_time_limit"]
+
+        # algorithms are checked during the fit method call,
+        # we need to know the data to tell if allgorithm is proper
+        if "algorithms" in kwargs:
+            self._algorithms = kwargs["algorithms"]
+
+        if "validation" in kwargs:
+            self._validation = kwargs["validation"]
+
+        if "train_ensemble" in kwargs:
+            self._train_ensemble = kwargs["train_ensemble"]
+
+        if "stack_models" in kwargs:
+            self._stack_models = kwargs["stack_models"]
+
+        if "explain_level" in kwargs:
+            self._explain_level = kwargs["explain_level"]
+
+        self._ml_task = None
+        if "ml_task" in kwargs:
+            self._ml_task = kwargs["ml_task"]
+
+        self._user_set_optimize_metric = None
+        if "optimize_metric" in kwargs:
+            self._user_set_optimize_metric = kwargs["optimize_metric"]
+
+        if "tuning_mode" in kwargs:
+            self.set_tuning_mode(kwargs["tuning_mode"])
+
+        self._verbose = True
+        if "verbose" is kwargs:
+            self._verbose = kwargs["verbose"]
+
+        self._seed = 1234
+        if "seed" in kwargs:
+            self._seed = kwargs["seed"]
+
+        self._tuner_params = {
+            "start_random_models": self._start_random_models,
+            "hill_climbing_steps": self._hill_climbing_steps,
+            "top_models_to_improve": self._top_models_to_improve,
+        }
         self._models = []  # instances of iterative learner framework or ensemble
 
         # it is instance of model framework or ensemble
         self._best_model = None
-        self._validation = validation
-        self.set_tuning_mode(tuning_mode)
-
-        self._algorithms = algorithms
-        self._verbose = verbose
+        self._verbose = True
 
         self._fit_time = None
         self._models_train_time = {}
-        self._threshold, self._metrics_details, self._max_metrics, self._confusion_matrix = (
-            None,
-            None,
-            None,
-            None,
-        )
-        self._seed = seed
-        self._user_set_optimize_metric = optimize_metric
-        self._ml_task = ml_task
+        self._threshold = None
+        self._metrics_details = None
+        self._max_metrics = None
+        self._confusion_matrix = None
 
         self._X_train_path, self._y_train_path = None, None
         self._X_validation_path, self._y_validation_path = None, None
@@ -174,15 +270,17 @@ class AutoML:
         self._data_info = None
         self._model_paths = []
         self._stacked_models = None
-        self._explain_level = explain_level
-        self._results_path = results_path
+
         self._fit_level = None
         self._time_spend = {}
         self._start_time = time.time()  # it will be updated in `fit` method
 
-        if self._validation["validation_type"] != "kfold":
-            # stack only available of k-fold validation
-            self._stack = False
+        if self._validation["validation_type"] != "kfold" and self._stack_models:
+            print(
+                "Models cannot be stacked. Please set validation to k-fold to stack models."
+            )
+            # stacking only available of k-fold validation
+            self._stack_models = False
 
         # this should be last in the constrcutor
         # in case there is a dir, it might load models
@@ -277,7 +375,9 @@ class AutoML:
 
             models_map = {}
             for model_path in self._model_paths:
-                if model_path.endswith("Ensemble") or model_path.endswith("Ensemble_Stacked"):
+                if model_path.endswith("Ensemble") or model_path.endswith(
+                    "Ensemble_Stacked"
+                ):
                     ens = Ensemble.load(model_path, models_map)
                     self._models += [ens]
                     models_map[ens.get_name()] = ens
@@ -368,7 +468,7 @@ class AutoML:
                 - self._time_spend["simple_algorithms"]
                 - self._time_spend["default_algorithms"]
             )
-            if self._stack:
+            if self._stack_models:
                 tt *= (
                     0.6
                 )  # leave some time for stacking (approx. 40% for stacking of time left)
@@ -384,7 +484,7 @@ class AutoML:
                 - self._time_spend["default_algorithms"]
                 - self._time_spend["not_so_random"]
             )
-            if self._stack:
+            if self._stack_models:
                 tt *= (
                     0.4
                 )  # leave some time for stacking (approx. 60% for stacking of time left)
@@ -392,7 +492,7 @@ class AutoML:
             tt /= k_folds  # time is per learner (per fold)
             return tt
 
-        if self._stack and self._fit_level == "stack":
+        if self._stack_models and self._fit_level == "stack":
             tt = time_left
             tt /= tune_algs_cnt  # give time equally for each algorithm
             tt /= k_folds  # time is per learner (per fold)
@@ -409,7 +509,7 @@ class AutoML:
             {
                 "learner_time_limit": self._get_learner_time_limit(
                     params["learner"]["model_type"]
-                ),  # self._time_limit,
+                ),
                 "min_steps": params["additional"].get("min_steps"),
             }
         )
@@ -430,7 +530,7 @@ class AutoML:
 
         if self._enough_time_to_train(mf.get_type()):
 
-            #self.verbose_print(params["name"] + " training start ...")
+            # self.verbose_print(params["name"] + " training start ...")
             logger.info(
                 f"Train model #{len(self._models)+1} / Model name: {params['name']}"
             )
@@ -494,7 +594,7 @@ class AutoML:
                 - self._time_spend["simple_algorithms"]
                 - self._time_spend["default_algorithms"]
             )
-            if self._stack:
+            if self._stack_models:
                 time_should_use *= 0.6  # leave time for stacking
             if self._hill_climbing_steps > 0:
                 time_should_use /= 2.0  # leave time for hill-climbing
@@ -518,7 +618,7 @@ class AutoML:
                 - self._time_spend["default_algorithms"]
                 - self._time_spend["not_so_random"]
             )
-            if self._stack:
+            if self._stack_models:
                 time_should_use *= 0.4  # leave time for stacking
 
             if (
@@ -600,8 +700,12 @@ class AutoML:
             oof.columns = [f"{m.get_name()}_{c}" for c in cols]
             all_oofs += [oof]
 
+        org_index = X.index.copy()
+        X.reset_index(drop=True, inplace=True)
         X_stacked = pd.concat(all_oofs + [X], axis=1)
 
+        X_stacked.index = org_index.copy()
+        X.index = org_index.copy()
         return X_stacked
 
     def stack_models(self):
@@ -621,7 +725,7 @@ class AutoML:
                 continue
             ds = ldb[ldb.model_type == model_type].copy()
             ds.sort_values(by="metric_value", inplace=True)
-            
+
             for n in list(ds.name.iloc[:models_limit].values):
                 self._stacked_models += [models_map[n]]
 
@@ -631,16 +735,16 @@ class AutoML:
         ]
 
     def stacked_ensemble_step(self):
-        #print("Stacked models ....")
+        # print("Stacked models ....")
         # do we have enough models?
         if len(self._models) < 5:
             return
         # do we have time?
-        #if self._total_time_limit is not None:
-        #    time_left = self._total_time_limit - (time.time() - self._start_time)
-        #    # we need at least 60 seconds to do anything
-        #    if time_left < 60:
-        #        return
+        if self._total_time_limit is not None:
+            time_left = self._total_time_limit - (time.time() - self._start_time)
+            # we need at least 60 seconds to do anything
+            if time_left < 60:
+                return
 
         # read X directly from parquet
         X = pd.read_parquet(self._X_train_path)
@@ -660,21 +764,17 @@ class AutoML:
 
         # resue old params
         for m in self._stacked_models:
-            #print(m.get_type())
-            # use only Xgboost, LightGBM and CatBoost as stacked models 
-            if m.get_type() not in [
-                "Xgboost",
-                "LightGBM",
-                "CatBoost"
-            ]:
+            # print(m.get_type())
+            # use only Xgboost, LightGBM and CatBoost as stacked models
+            if m.get_type() not in ["Xgboost", "LightGBM", "CatBoost"]:
                 continue
 
-            params = copy.deepcopy(m.params)    
+            params = copy.deepcopy(m.params)
             params["validation"]["X_train_path"] = X_train_stacked_path
 
             params["name"] = params["name"] + "_Stacked"
             params["is_stacked"] = True
-            #print(params)
+            # print(params)
 
             if "model_architecture_json" in params["learner"]:
                 # the new model will be created with wider input size
@@ -691,7 +791,7 @@ class AutoML:
                 if scale is not None:
                     for col in added_columns:
                         params["preprocessing"]["columns_preprocessing"][col] = [scale]
-            
+
             self.train_model(params)
 
     def _set_ml_task(self, y):
@@ -832,6 +932,9 @@ class AutoML:
 
         X_train.to_parquet(self._X_train_path, index=False)
 
+        if self._ml_task == MULTICLASS_CLASSIFICATION:
+            y_train = y_train.astype(str)
+
         pd.DataFrame({"target": y_train}).to_parquet(self._y_train_path, index=False)
 
         self._validation["X_train_path"] = self._X_train_path
@@ -882,9 +985,9 @@ class AutoML:
         """
         try:
 
-            #if self._best_model is not None:
-            #    print("Best model is already set, no need to run fit. Skipping ...")
-            #    return
+            if self._best_model is not None:
+                print("Best model is already set, no need to run fit. Skipping ...")
+                return
 
             self._start_time = time.time()
 
@@ -923,7 +1026,6 @@ class AutoML:
             self._time_spend = {}
             self._time_start = {}
 
-            
             # 1. Check simple algorithms
             self._fit_level = "simple_algorithms"
             start = time.time()
@@ -956,7 +1058,6 @@ class AutoML:
             for params in tuner.get_hill_climbing_params(self._models):
                 self.train_model(params)
             self._time_spend["hill_climbing"] = np.round(time.time() - start, 2)
-            
 
             # 5. Ensemble unstacked models
             self._fit_level = "ensemble_unstacked"
@@ -965,8 +1066,7 @@ class AutoML:
             self.ensemble_step()
             self._time_spend["ensemble_unstacked"] = np.round(time.time() - start, 2)
 
-            
-            if self._stack:
+            if self._stack_models:
                 # 6. Stack best models
                 self._fit_level = "stack"
                 start = time.time()
@@ -1053,6 +1153,7 @@ class AutoML:
         if self._best_model._is_stacked:
             self.stack_models()
             X_stacked = self.get_stacked_data(X, mode="predict")
+
             if self._best_model.get_type() == "Ensemble":
                 # Ensemble is using both original and stacked data
                 predictions = self._best_model.predict(X, X_stacked)
